#include <functional>
#include <iostream>
#include <pthread.h>
#include <sys/syscall.h>
#include <getopt.h>
#include <chrono>
#include <asm/types.h>
#include <assert.h>
#include <fcntl.h>
#include <dirent.h>
#include <malloc.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <sys/ioctl.h>
#include <sys/mman.h>
#include <sys/stat.h>
#include <sys/types.h>
#include <time.h>
#include <unistd.h>
#include <linux/stddef.h>
#include <linux/videodev2.h>
#include <errno.h>
#include <opencv2/opencv.hpp>

using namespace cv;
using namespace std;

const string winname = "Usb Camera";

unsigned long long nanoTime()
{
    struct timespec t;

    t.tv_sec = t.tv_nsec = 0;
    clock_gettime(CLOCK_MONOTONIC, &t);

    return (unsigned long long)(t.tv_sec) * 1000000000 + t.tv_nsec;
}

static void usage(char **argv)
{
    printf(
        "Usage: %s [Options]\n\n"
        "Options:\n"
        "-d, --device                 Usb camera device, 0, 1, 2 ...\n"
        "-w, --width                  Destination images's width\n"
        "-h, --height                 Destination images's height\n"
        "\n",
        argv[0]);
}

static const char *short_options = "d:w:h:";

static struct option long_options[] = {
    {"device", required_argument, NULL, 'd'},
    {"width", required_argument, NULL, 'w'},
    {"height", required_argument, NULL, 'h'},
    {NULL, 0, NULL, 0}
};

int camera(int dev, __u32 w, __u32 h)
{
    int ret = 0;
    size_t len;
    VideoCapture cap;
    Mat mat;

    /* TODO: Display window name */
    namedWindow(winname, WINDOW_NORMAL);
    /* TODO: Set window x and y position */
    moveWindow(winname, 0, 0);
    /* TODO: Set window width and height */
    resizeWindow(winname, w, h);
    /* TODO: Open camera device */
    cap.open(dev);
    /* TODO: Set frame format, width and height  */
    cap.set(CAP_PROP_FRAME_WIDTH, w);
    cap.set(CAP_PROP_FRAME_HEIGHT, h);

    while (1) {
	/* TODO: Capture picture */
	cap >> mat;
	printf("%lu, %lu, %lu\n", mat.cols, mat.rows, mat.elemSize1());
	/* TODO: Show image in windows */
        imshow(winname, mat);

	waitKey(1);
        if (getWindowProperty(winname, WND_PROP_AUTOSIZE) != WINDOW_NORMAL)
           break;
    }
	
exit:
    /* TODO: Close camera device */
    cap.release();

    /* TODO: Destroy window */
    destroyAllWindows();
    return ret;
}

int main(int argc, char **argv)
{
    int c, r;
    int dev = -1;
    char index = -1;

    __u32 width = 640, height = 480;

    while((c = getopt_long(argc, argv, short_options, long_options, NULL)) != -1) {
        switch (c) {
        case 'd':
            dev = atoi(optarg);
            break;
        case 'w':
            width = atoi(optarg);
            break;
        case 'h':
            height = atoi(optarg);
            break;
        default:
            usage(argv);
            return 0; 
        }
    }

    return camera(dev, width, height);
}
