#include <functional>
#include <iostream>
#include <pthread.h>
#include <sys/syscall.h>
#include <getopt.h>
#include <chrono>
#include <asm/types.h>
#include <assert.h>
#include <fcntl.h>
#include <dirent.h>
#include <malloc.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <sys/ioctl.h>
#include <sys/mman.h>
#include <sys/stat.h>
#include <sys/types.h>
#include <time.h>
#include <unistd.h>

#include <linux/stddef.h>
#include <linux/videodev2.h>
#include <errno.h>

#include <common.hpp>

using namespace std;

unsigned long long nanoTime()
{
    struct timespec t;

    t.tv_sec = t.tv_nsec = 0;
    clock_gettime(CLOCK_MONOTONIC, &t);

    return (unsigned long long)(t.tv_sec) * 1000000000 + t.tv_nsec;
}

static void usage(char **argv)
{
    printf(
        "Usage: %s {usb|mipi|hdmirx} [Options]\n\n"
        "Options:\n"
        "-d, --device                 Usb camera device: 0, 1, 2 ...\n"
        "-w, --width                  Destination images's width\n"
        "-h, --height                 Destination images's height\n"
        "\n",
        argv[0]);
}

static const char *short_options = "d:w:h:";

static struct option long_options[] = {
    {"device", required_argument, NULL, 'd'},
    {"width", required_argument, NULL, 'w'},
    {"height", required_argument, NULL, 'h'},
    {NULL, 0, NULL, 0}
};

int main(int argc, char **argv)
{
	int c, ret, frames;
	char index = -1;
	CAM_TYPE type;
	__u32 width = 0, height = 0;
	bool is_cmabuffer = true;
	int dev = -1;
	DrmBuffer *buf = NULL;
	v4l2_camera *camera = NULL;
	if (argc < 2) {
		usage(argv);
		return -EINVAL;
	} else if (strcmp(argv[1], "usb") == 0) {
		type = CAM_TYPE_USB;
	} else if (strcmp(argv[1], "mipi") == 0) {
		type = CAM_TYPE_MIPI;
	} else if (strcmp(argv[1], "hdmirx") == 0) {
		type = CAM_TYPE_HDMIRX;
	} else {
		usage(argv);
		return -EINVAL;
	}

	while((c = getopt_long(argc - 1, &argv[1], short_options, long_options, NULL)) != -1) {
		switch (c) {
		case 'd':
			dev = atoi(optarg);
			break;
		case 'w':
			width = atoi(optarg);
			break;
		case 'h':
			height = atoi(optarg);
			break;
		default:
			usage(argv);
			return 0; 
		}
	}

	printf("[%s camera]: dev: /dev/video%d, width %u, height %u\n", argv[1], dev, width, height);
	camera = new v4l2_camera(dev, type, width, height, is_cmabuffer);

	ret = camera->start();
	if (ret < 0)
		return ret;

	while (1) {
		buf = camera->get_frame();
		if (buf == NULL)
			break;
		printf("Get frame: size %lu, viraddr %p, physaddr 0x%lx, fd %d\n", buf->size, buf->virt, buf->phys, buf->fd);
	}

	return 0;
}
